/*******************************************************************************
 * Copyright (c) 2004, 2006 Sybase, Inc. and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sybase, Inc. - initial API and implementation
 *******************************************************************************/

package org.eclipse.jst.jsf.facesconfig.common.utils;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jst.j2ee.web.componentcore.util.WebArtifactEdit;
import org.eclipse.jst.jsf.facesconfig.common.IFileFolderConstants;

public class WebrootUtil {

	/**
	 * get the webpath for the project path. The project path is something like
	 * "/projectname/webroot/filename.jsp", or "/projectname/webroot/folder".
	 * The project information should be removed from project path, e.g,
	 * "/filename.jsp" or "/folder/*";
	 * 
	 * @param path
	 * @return
	 */
	public static String getWebPath(IPath path) {
		String strWebrootPath = "";
		IProject project = WorkspaceUtil.getProjectFor(path);
		IPath webContentPath = getWebContentPath(project);
		if (webContentPath != null && webContentPath.isPrefixOf(path)) {
			int start = path.matchingFirstSegments(webContentPath);
			String[] segments = path.segments();
			for (int i = start, n = path.segmentCount(); i < n; i++) {
				strWebrootPath = strWebrootPath
						+ IFileFolderConstants.PATH_SEPARATOR + segments[i];
			}
		}
		return strWebrootPath;
	}

	/**
	 * To see if a resource is under the webcontent folder.
	 * 
	 * @param resource
	 * @return
	 */
	public static boolean isUnderWebContentFolder(IResource resource) {
		IPath webContentPath = getWebContentPath(resource.getProject());
		if (webContentPath != null) {
			return webContentPath.isPrefixOf(resource.getFullPath());
		}
		return true;
	}

	public static IPath getWebContentPath(IProject project) {
		if (project != null) {
			WebArtifactEdit web = null;
			try {
				web = WebArtifactEdit.getWebArtifactEditForRead(project);
				if (web != null) {
					IPath webxml = web.getDeploymentDescriptorPath();
					IPath webContentPath = webxml.removeLastSegments(2);
					return webContentPath;
				}
			} finally {
				if (web != null) {
					web.dispose();
				}
			}
		}
		return null;
	}

	public static IFolder getWebContentFolder(IProject project) {
		IPath webContentPath = getWebContentPath(project);
		if (webContentPath != null) {
			IFolder folder;
			if (webContentPath.segmentCount() > 1) {
				folder = project.getFolder(webContentPath
						.removeFirstSegments(1));
			} else {
				folder = ResourcesPlugin.getWorkspace().getRoot().getFolder(
						webContentPath);
			}
			return folder;
		}
		return null;
	}

	/**
	 * return the depth of webcontent folder. For example, if the webcontent
	 * folder path is /projectname/webContent, then return 2, if it's
	 * /projectname/a/webContent, then return 3.
	 * 
	 * @param project
	 * @return the depth of webcontent folder
	 */
	public static int getWebContentFolderDepth(IProject project) {
		if (project != null) {
			IPath webContentPath = getWebContentPath(project);
			if (webContentPath != null) {
				return webContentPath.segmentCount();
			}
		}
		// default to 2
		return 2;
	}

	/**
	 * determine the path of web file is valid or not
	 * 
	 * @param path -
	 *            the path of web file
	 * @return - true - valid web file
	 */
	public static boolean isValidWebFile(IPath path) {
		if (path.getFileExtension() != null) {
			if (path.getFileExtension().equalsIgnoreCase(
					IFileFolderConstants.EXT_JSP)
					|| path.getFileExtension().equalsIgnoreCase(
							IFileFolderConstants.EXT_JSF)
					|| path.getFileExtension().equalsIgnoreCase(
							IFileFolderConstants.EXT_JSV)
					|| path.getFileExtension().equalsIgnoreCase(
							IFileFolderConstants.EXT_JTPL)
					|| path.getFileExtension().equalsIgnoreCase(
							IFileFolderConstants.EXT_JSPX)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * get the webpath for the project path. The project path is something like
	 * "/projectname/webroot/filename.jsp", or "/projectname/webroot/folder".
	 * The project information should be removed from project path, e.g,
	 * "/filename.jsp" or "/folder/*";
	 * 
	 * @param strPath -
	 *            the project path
	 * @return - web path remove from "/projectname/webroot"
	 * @deprecated use getWebPath(IPath path) instead.
	 */
	public static String getWebPath(String strPath) {
		String strWebrootPath = "";
		if (strPath != null) {
			IPath path = new Path(strPath);
			return getWebPath(path);
		}
		return strWebrootPath;
	}

	public static String getPageNameFromWebPath(String strWebPath) {
		String pageName = strWebPath;

		if (pageName.startsWith(IFileFolderConstants.PATH_SEPARATOR)) {
			pageName = pageName.substring(1);
		}
		String jspExt = IFileFolderConstants.DOT + IFileFolderConstants.EXT_JSP;
		String jsfExt = IFileFolderConstants.DOT + IFileFolderConstants.EXT_JSF;
		String jsvExt = IFileFolderConstants.DOT + IFileFolderConstants.EXT_JSV;
		String jtplExt = IFileFolderConstants.DOT
				+ IFileFolderConstants.EXT_JTPL;
		String jspxExt = IFileFolderConstants.DOT
				+ IFileFolderConstants.EXT_JSPX;
		if (pageName.endsWith(jspExt)) {
			pageName = pageName.substring(0, pageName.length()
					- jspExt.length());
		} else if (pageName.endsWith(jsfExt)) {
			pageName = pageName.substring(0, pageName.length()
					- jsfExt.length());
		} else if (pageName.endsWith(jsvExt)) {
			pageName = pageName.substring(0, pageName.length()
					- jsvExt.length());
		} else if (pageName.endsWith(jtplExt)) {
			pageName = pageName.substring(0, pageName.length()
					- jtplExt.length());
		} else if (pageName.endsWith(jspxExt)) {
			pageName = pageName.substring(0, pageName.length()
					- jspxExt.length());
		}

		return pageName;
	}
}
