/*******************************************************************************
 * Copyright (c) 2006 Oracle Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Cameron Bateman/Oracle - initial API and implementation
 *    
 ********************************************************************************/

package org.eclipse.jst.jsf.validation.internal.el.operators;

import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.jst.jsf.core.internal.types.TypeTransformer;
import org.eclipse.jst.jsf.core.internal.types.FloatLiteralType;
import org.eclipse.jst.jsf.core.internal.types.IntegerLiteralType;
import org.eclipse.jst.jsf.core.internal.types.LiteralType;
import org.eclipse.jst.jsf.core.internal.types.NullLiteralType;
import org.eclipse.jst.jsf.core.internal.types.StringLiteralType;
import org.eclipse.jst.jsf.core.internal.types.TypeCoercer;
import org.eclipse.jst.jsf.core.internal.types.TypeCoercionException;
import org.eclipse.jst.jsf.core.internal.types.TypeConstants;
import org.eclipse.jst.jsf.core.internal.types.ValueType;
import org.eclipse.jst.jsf.validation.internal.el.diagnostics.DiagnosticFactory;

/**
 * Represents the EL unary operator '-' on a ValueType 
 * as described by JSP.2.3.5.4
 * 
 * 
 * @author cbateman
 *
 */
/*package*/class MinusUnaryOperator extends UnaryOperator 
{
    public Diagnostic validate(ValueType type)
    {
        // must coerce to numeric type
        try
        {
            // if coerceTypeNumber doesn't throw an exception, then
            // give the benefit of the doubt
            // TODO:we are, for now lumping in the indeterminate case here
            String coercedType =
                TypeCoercer.
                    coerceToNumber(TypeTransformer.
                            transformBoxPrimitives(type.getSignature()));
            
            // if it's string and we have the value, we can determine for
            // sure whether or not it's coercable to a number
            // per JSP.2.3.5.4 step 3
            if (type instanceof StringLiteralType)
            {
                String literalValue = ((LiteralType)type).getLiteralValue();
                if (literalValue.indexOf('.') > -1
                        || literalValue.indexOf('e') > -1
                        || literalValue.indexOf('E') > -1)
                {
                    // if it coerces to double, then it's a double
                    ((LiteralType)type).coerceToNumber(Double.class);
                    coercedType = TypeConstants.TYPE_BOXED_DOUBLE;
                }
                else
                {
                    // if it coerces to long, then it's a long
                    ((LiteralType)type).coerceToNumber(Long.class);
                    coercedType = TypeConstants.TYPE_BOXED_LONG;
                }
            }
            
            if (type instanceof NullLiteralType)
            {
                // null always coerces to 0L on this operator
                return DiagnosticFactory.
                    create_UNARY_OP_MINUS_ON_NULL_ALWAYS_ZERO();
            }
            
            if (coercedType != null)
            {
                return Diagnostic.OK_INSTANCE;
            }

            return DiagnosticFactory.
                create_UNARY_OP_STRING_CONVERSION_NOT_GUARANTEED
                    ("unary minus");
        }
        catch (TypeCoercionException tce)
        {
            return DiagnosticFactory.create_UNARY_OP_COULD_NOT_MAKE_NUMERIC_COERCION("unary minus");
        }
    }
    
    /**
     * Based on JSP.2.3.5.4
     * 
     * @param type
     * @return type of type after a minus is applied or null if unknown
     */
    public ValueType performOperation(ValueType type)
    {
        try
        {
            final String  boxedType = 
                TypeTransformer.transformBoxPrimitives(type.getSignature());
            // check for valid type coercion 
            TypeCoercer.coerceToNumber(boxedType);

            // if it's string and we have the value, we can determine for
            // sure whether or not it's coercable to a number
            // per JSP.2.3.5.4 step 3
            if (type instanceof StringLiteralType)
            {
                String literalValue = ((LiteralType)type).getLiteralValue();
                if (literalValue.indexOf('.') > -1
                        || literalValue.indexOf('e') > -1
                        || literalValue.indexOf('E') > -1)
                {
                    // if it coerces to double, then it's a double
                    Number value = ((LiteralType)type).coerceToNumber(Double.class);
                    return new FloatLiteralType(-1 * value.doubleValue());
                }

                // if it coerces to long, then it's a long
                Number value = ((LiteralType)type).coerceToNumber(Long.class);
                return new IntegerLiteralType(-1 * value.longValue());
            }
            
            if (type instanceof NullLiteralType)
            {
                // null always coerces to 0L on this operator
                return new IntegerLiteralType(0L);
            }

            // JSP.2.3.5.4
            // big integer and big decimal retain type
            // all numeric types retain type
            if (TypeCoercer.typeIsNumeric(boxedType))
            {
                // integer and float literals are special because -1 or -1.0
                // is syntically minusOp(1) and minusOp(1.0)
                if (type instanceof IntegerLiteralType)
                {
                    return new IntegerLiteralType(-1 * ((IntegerLiteralType)type).coerceToNumber(Long.class).longValue());
                }
                else if (type instanceof FloatLiteralType)
                {
                    return new FloatLiteralType(-1 * ((FloatLiteralType)type).coerceToNumber(Double.class).doubleValue());
                }
                return type;
            }
           
            // all other cases, return null
            // even is type represents a String, without it's value, we have
            // no idea how to coerce it without it's value
            // fall through and return null
        }
        catch (TypeCoercionException tce)
        {
            // do nothing, fall through and return null
        }
        
        return null;
    }

}
